@@ -21,27 +21,19 @@ import java.util.BitSet;
 import java.util.HashMap;
 import java.util.NoSuchElementException;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.commons.pool.KeyedObjectPool;
 import org.apache.commons.pool.KeyedPoolableObjectFactory;
-import org.apache.commons.pool.TestKeyedObjectPool;
 import org.apache.commons.pool.TestBaseKeyedObjectPool;
 
 /**
  * @author Rodney Waldhoff
- * @version $Revision: 605736 $ $Date: 2007-12-19 16:25:47 -0700 (Wed, 19 Dec 2007) $
+ * @version $Revision: 958393 $ $Date: 2010-06-27 12:32:43 -0400 (Sun, 27 Jun 2010) $
  */
 public class TestStackKeyedObjectPool extends TestBaseKeyedObjectPool {
     public TestStackKeyedObjectPool(String testName) {
         super(testName);
     }
 
-    public static Test suite() {
-        return new TestSuite(TestStackKeyedObjectPool.class);
-    }
-
     protected KeyedObjectPool makeEmptyPool(int mincapacity) {
         StackKeyedObjectPool pool = new StackKeyedObjectPool(new SimpleFactory(),mincapacity);
         return pool;
@@ -118,6 +110,24 @@ public class TestStackKeyedObjectPool extends TestBaseKeyedObjectPool {
             assertEquals((i < 8 ? i+1 : 8),pool.getNumIdle(""));
         }
     }
+    
+    /**
+     * Verifies maxSleeping contract: When returnObject triggers maxSleeping exceeded,
+     * the bottom (oldest) instance in the pool is destroyed to make room for the newly
+     * returning instance, which is pushed onto the idle object stack.
+     */
+    public void testRemoveOldest() throws Exception {
+        pool._maxSleeping = 2;
+        Object obj0 = pool.borrowObject("");
+        Object obj1 = pool.borrowObject("");
+        Object obj2 = pool.borrowObject("");
+        pool.returnObject("", obj0); // Push 0 onto bottom of stack
+        pool.returnObject("", obj1); // Push 1
+        pool.returnObject("", obj2); // maxSleeping exceeded -> 0 destroyed, 2 pushed
+        assertEquals("2", pool.borrowObject("")); // 2 was pushed on top
+        assertEquals("1", pool.borrowObject("")); // 1 still there
+        assertEquals("3", pool.borrowObject("")); // New instance created (0 is gone)
+    }
 
     public void testPoolWithNullFactory() throws Exception {
         KeyedObjectPool pool = new StackKeyedObjectPool(10);
@@ -281,13 +291,13 @@ public class TestStackKeyedObjectPool extends TestBaseKeyedObjectPool {
         }
         // 3, 9, 15 fail passivation.  
         assertEquals(7,pool.getNumIdle());
-        assertEquals(new Integer(19), (Integer) pool.borrowObject("key"));
-        assertEquals(new Integer(17), (Integer) pool.borrowObject("key"));
-        assertEquals(new Integer(13), (Integer) pool.borrowObject("key"));
-        assertEquals(new Integer(11), (Integer) pool.borrowObject("key"));
-        assertEquals(new Integer(7), (Integer) pool.borrowObject("key"));
-        assertEquals(new Integer(5), (Integer) pool.borrowObject("key"));
-        assertEquals(new Integer(1), (Integer) pool.borrowObject("key"));   
+        assertEquals(new Integer(19), pool.borrowObject("key"));
+        assertEquals(new Integer(17), pool.borrowObject("key"));
+        assertEquals(new Integer(13), pool.borrowObject("key"));
+        assertEquals(new Integer(11), pool.borrowObject("key"));
+        assertEquals(new Integer(7), pool.borrowObject("key"));
+        assertEquals(new Integer(5), pool.borrowObject("key"));
+        assertEquals(new Integer(1), pool.borrowObject("key"));   
     }
 
     class SimpleFactory implements KeyedPoolableObjectFactory {
